"""
The python source code.
"""

from SpAlgo._array import Array
from SpAlgo._matrix import Matrix
from SpAlgo._sequence import MaxSubseq
from SpAlgo._knapsack._knapsack import Knapsack

__version__ = "0.2.1"
__all__ = ['Matrix', 'MaxSubseq', 'Array', 'Knapsack']
